@@ -13,6 +13,7 @@ from allure_commons._allure import attach
 from allure_commons._allure import manual
 from allure_commons.types import Severity as severity_level
 from allure_commons.types import AttachmentType as attachment_type
+from allure_commons.types import ParameterMode as parameter_mode
 
 
 __all__ = [
@@ -32,10 +33,11 @@ __all__ = [
     'link',
     'issue',
     'testcase',
+    'manual',
     'step',
     'dynamic',
     'severity_level',
     'attach',
     'attachment_type',
-    'manual'
+    'parameter_mode'
 ]
