@@ -1,10 +1,12 @@
 # -*- coding: utf-8 -*-
 
 import os
+import string
 import sys
 import six
 import time
 import uuid
+import json
 import socket
 import inspect
 import hashlib
@@ -12,6 +14,7 @@ import platform
 import threading
 import traceback
 import collections
+
 from functools import partial
 
 
@@ -30,10 +33,10 @@ def getargspec(func):
 
     Like inspect.getargspec but supports functools.partial as well.
     """
-    # type: (Any) -> Any
+    # noqa: E731 type: (Any) -> Any
     if inspect.ismethod(func):
         func = func.__func__
-    parts = 0, ()  # type: Tuple[int, Tuple[unicode, ...]]
+    parts = 0, ()  # noqa: E731 type: Tuple[int, Tuple[unicode, ...]]
     if type(func) is partial:
         keywords = func.keywords
         if keywords is None:
@@ -108,6 +111,7 @@ def escape_non_unicode_symbols(item):
             return s
         except UnicodeDecodeError:
             return repr(s)[1:-1]
+
     return ''.join(map(escape_symbol, item))
 
 
@@ -221,6 +225,9 @@ def func_parameters(func, *args, **kwargs):
     >>> args_kwargs(1, 2, 5, 6)
     [('a', '1'), ('b', '2'), ('c', '5'), ('d', '6')]
 
+    >>> args_kwargs(1, b=2)
+    [('a', '1'), ('b', '2'), ('c', '3'), ('d', '4')]
+
     >>> @helper
     ... def varargs(*a):
     ...     pass
@@ -304,7 +311,7 @@ def func_parameters(func, *args, **kwargs):
     args_dict = dict(zip(arg_spec.args, args))
 
     if arg_spec.defaults:
-        kwargs_defaults_dict = dict(zip(arg_spec.args[len(args):], arg_spec.defaults))
+        kwargs_defaults_dict = dict(zip(arg_spec.args[-len(arg_spec.defaults):], arg_spec.defaults))
         parameters.update(kwargs_defaults_dict)
 
     if arg_spec.varargs:
@@ -386,3 +393,65 @@ def format_exception(etype, value):
     "AssertionError: \\nExpected:...but:..."
     """
     return '\n'.join(format_exception_only(etype, value)) if etype or value else None
+
+
+def get_testplan():
+    planned_tests = []
+    file_path = os.environ.get("ALLURE_TESTPLAN_PATH")
+
+    if file_path and os.path.exists(file_path):
+        with open(file_path, 'r') as plan_file:
+            plan = json.load(plan_file)
+            planned_tests = plan.get("tests", [])
+
+    return planned_tests
+
+
+class SafeFormatter(string.Formatter):
+    """
+    Format string safely - skip any non-passed keys
+    >>> f = SafeFormatter().format
+
+    Make sure we don't broke default formatting behaviour
+    >>> f("literal string")
+    'literal string'
+    >>> f("{expected.format}", expected=str)
+    "<method 'format' of 'str' objects>"
+    >>> f("{expected[0]}", expected=["value"])
+    'value'
+    >>> f("{expected[0]}", expected=123)
+    Traceback (most recent call last):
+    ...
+    TypeError: 'int' object is not subscriptable
+    >>> f("{expected[0]}", expected=[])
+    Traceback (most recent call last):
+    ...
+    IndexError: list index out of range
+    >>> f("{expected.format}", expected=int)
+    Traceback (most recent call last):
+    ...
+    AttributeError: type object 'int' has no attribute 'format'
+
+    Check that unexpected keys do not cause some errors
+    >>> f("{expected} {unexpected}", expected="value")
+    'value {unexpected}'
+    >>> f("{unexpected[0]}", expected=["value"])
+    '{unexpected[0]}'
+    >>> f("{unexpected.format}", expected=str)
+    '{unexpected.format}'
+    """
+
+    class SafeKeyOrIndexError(Exception):
+        pass
+
+    def get_field(self, field_name, args, kwargs):
+        try:
+            return super().get_field(field_name, args, kwargs)
+        except self.SafeKeyOrIndexError:
+            return "{" + field_name + "}", field_name
+
+    def get_value(self, key, args, kwargs):
+        try:
+            return super().get_value(key, args, kwargs)
+        except (KeyError, IndexError):
+            raise self.SafeKeyOrIndexError()
